/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.perfs;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.facility.naming.gobid.binder.LBinderManager;
import org.objectweb.jorm.facility.naming.gobid.generator.LGeneratorManager;
import org.objectweb.jorm.facility.naming.gobid.pnc.LLPncManager;
import org.objectweb.jorm.facility.naming.usid.StrBinder;
import org.objectweb.jorm.facility.meminst.manager.basic.api.Manager;
import org.objectweb.jorm.facility.meminst.manager.basic.genclass.java.util.SetManager;
import org.objectweb.jorm.runtime.Mapper;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.transaction.ra.gen.FactoryImpl;
import org.objectweb.transaction.api.ConnectionFactoryWrapper;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.HandlerFactory;
import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.wrapper.common.Configurable;
import org.objectweb.util.monolog.file.monolog.PropertiesConfAccess;

import java.util.Properties;
import java.util.Enumeration;
import java.io.FileInputStream;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;
import javax.naming.Context;


/**
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */
public class JormBoot {

	public static final String LOGGER_FACTORY_CLASS_NAME
		= "loggerFactoryClassName";

	public static final String RESOURCE_MCF_CLASSNAME
		= "resource.mcf.classname";

	public String mapperName = null;
	public String lGeneratorMappingClassName = null;
	public String lBinderMappingClassName = null;
	public String llPncMappingClassName = null;
	public String setMappingClassName = null;

	public PMapper mapper = null;

	public PClassMapping llPncMapping = null;

	public PClassMapping lBinderMapping = null;

	public ConnectionFactory connectionFactory = null;

	public Object conn = null;

	/**
	 * url of the jdbc driver
	 */
	public String url = null;

	/**
	 * user used to access the jdbc database
	 */
	public String user = null;

	/**
	 * password used to access the jdbc database associated to the user
	 */
	public String passwd = null;

	public LoggerFactory loggerFactory = null;

	public Logger logger = null;

	public void initJorm(byte cleanAction) throws Exception {
		mapper = new Mapper();
		mapper.setMapperName(mapperName);
		((Loggable) mapper).setLoggerFactory(loggerFactory);
		logger.log(BasicLevel.INFO, "Mapper " + mapperName
			+ "instancied ant initialised");

		// LGenerator initialization
		StrBinder bsoflg = new StrBinder();
		PClassMapping lgm = (PClassMapping) newInstance(lGeneratorMappingClassName);
		lgm.setPBinder(bsoflg);
		lgm.setMIManager(lgm);
		((LGeneratorManager) lgm).setConnectionFactory(connectionFactory);
		((LGeneratorManager) lgm).setLoggerFactory(loggerFactory);
		bsoflg.setPClassMapping(lgm);
		mapper.map(conn, lgm, cleanAction);
		logger.log(BasicLevel.INFO, "LGenerator objects mapped");

		// LBinder initialization
		StrBinder bsoflb = new StrBinder();
		lBinderMapping = (PClassMapping) newInstance(lBinderMappingClassName);
		lBinderMapping.setPBinder(bsoflb);
		lBinderMapping.setMIManager(lBinderMapping);
		((LBinderManager) lBinderMapping).setConnectionFactory(connectionFactory);
		((LBinderManager) lBinderMapping).setLoggerFactory(loggerFactory);
		bsoflb.setPClassMapping(lBinderMapping);
		mapper.map(conn, lBinderMapping, cleanAction);
		logger.log(BasicLevel.INFO, "LBinder objects mapped");

		// Set initialization
		StrBinder bsofs = new StrBinder();
		PClassMapping sm = (PClassMapping) newInstance(setMappingClassName);
		((Loggable) sm).setLoggerFactory(loggerFactory);
		SetManager setmanager = new SetManager();
		setmanager.setConnectionFactory(connectionFactory);
		setmanager.setLoggerFactory(loggerFactory);
		sm.setMIManager(setmanager);
		sm.setPBinder(bsofs);
		bsofs.setPClassMapping(sm);
		sm.setGenClassMapping(lBinderMapping);
		((Loggable) sm).setLogger(
			loggerFactory.getLogger(sm.getClass().getName()));
		logger.log(BasicLevel.INFO, "Set objects mapped");


		// LLPnc initialization
		StrBinder bsofpnc = new StrBinder();
		llPncMapping = (PClassMapping) newInstance(llPncMappingClassName);
		llPncMapping.setPBinder(bsofpnc);
		llPncMapping.setMIManager(llPncMapping);
		((LLPncManager) llPncMapping).setConnectionFactory(connectionFactory);
		((LLPncManager) llPncMapping).setLoggerFactory(loggerFactory);
		bsofpnc.setPClassMapping(llPncMapping);
		llPncMapping.setGenClassMapping("classBinders", sm);
		mapper.map(conn, llPncMapping, cleanAction);
		logger.log(BasicLevel.INFO, "PNCLong object mapped");

		// Links the pclassmapping of the LBinder objects to the pnc which manages
		// LGenerator references.
		lBinderMapping.setPNamingContext("generator", bsoflg, true);
		logger.log(BasicLevel.INFO, "Pnc of Generator linked");

		// Links the pclassmapping of the Set objects to the pnc which manages
		// LBinder references.
		sm.setPNamingContext(bsoflb, true);
		logger.log(BasicLevel.INFO, "Pnc of LBinder linked");

		// Links the pclassmapping of the LLPnc objects to the pnc which manages
		// Set references.
		llPncMapping.setPNamingContext("classBinders", bsofs, true);
		logger.log(BasicLevel.INFO, "Pnc of Set linked");

		logger.log(BasicLevel.DEBUG, "Init jorm OK");
	}

	public void initResourceAdapter(Properties prop) throws Exception {

		mapperName = prop.getProperty("mapper.name");
		lGeneratorMappingClassName =
			"org.objectweb.jorm.facility.naming.gobid.generator." + mapperName
			+ ".LGeneratorMapping";
		lBinderMappingClassName =
			"org.objectweb.jorm.facility.naming.gobid.binder." + mapperName
			+ ".LBinderMapping";
		llPncMappingClassName =
			"org.objectweb.jorm.facility.naming.gobid.pnc." + mapperName
			+ ".LLPncMapping";
		setMappingClassName =
			"org.objectweb.jorm.genclass." + mapperName
			+ ".lib." + upperFL(mapperName) + "GenClassMapping";


		//Instanciate the resource adapter
		String property = prop.getProperty(RESOURCE_MCF_CLASSNAME);
		if (property == null)
		    conn = new String();
		else {
		    Object ra =	newInstance(property);
		    logger.log(BasicLevel.DEBUG, "ra=" + ra);
		    
		    // configure it
		    Context ctx = properties2Context(prop);
		    ctx.bind("org.objectweb.util.monolog.api.logger",
			     loggerFactory.getLogger("ra."+mapperName));
		    ((FactoryImpl) ra).init(ctx);
		    
		    //Fetch the ConnectionFactory
		    if (ra instanceof ConnectionFactory)
			connectionFactory = (ConnectionFactory) ra;
		    else if (ra instanceof ConnectionFactoryWrapper)
			connectionFactory =
			    ((ConnectionFactoryWrapper) ra).getCCIConnectionFactory();
		    else
			throw new Exception("Impossible to use this resource adapter:"
					    + "not a ConnectionFactory or a ConnecitonFactoryWrapper");
		    
		    conn = connectionFactory.getConnection();
		    logger.log(BasicLevel.INFO, "connection factory=" + connectionFactory);
		}
	}

	public Connection getNewConnection() throws Exception {
		return connectionFactory.getConnection();
	}

	public void initLogSystem(Properties prop) throws Exception {
		String a = prop.getProperty(Configurable.LOG_CONFIGURATION_FILE, null);
		if (a!=null) {
			String b = prop.getProperty(
				Configurable.LOG_CONFIGURATION_FILE_USE_CLASSPATH, null);
			Properties p = new Properties();
			if (b==null && "true".equalsIgnoreCase(b)) {
				// Load from the classpath
				p.load(ClassLoader.getSystemResourceAsStream(a));
			}
			else {
				//Load from file system
				p.load(new FileInputStream(a));
			}
			// recall this method with the new properties
			initLogSystem(p);
		}
		else {
			a = prop.getProperty("log.config.classname", null);
			if (a==null) {
				throw new Exception(
					"Impossible to find the 'log.config.classname' property");
			}
			loggerFactory = (LoggerFactory) newInstance(a);
			PropertiesConfAccess.load(prop, loggerFactory,
				(HandlerFactory) loggerFactory, (LevelFactory) loggerFactory);
			logger = loggerFactory.getLogger("org.objectweb.jorm.examples.JormBoot");
			logger.log(BasicLevel.INFO, "Log system init");
		}
	}

	public Object newInstance(String classname) throws Exception {
		return Class.forName(classname).newInstance();
	}

	public final static Context properties2Context(Properties p)
		throws Exception {
		Context res = new org.objectweb.naming.ContextImpl();
		Enumeration e = p.keys();
		while(e.hasMoreElements()) {
			String key = (String) e.nextElement();
			res.rebind(key, p.getProperty(key, ""));
		}
		return res;
	}

	public static String upperFL(String word) {
		return word.substring(0,1).toUpperCase()
			+ word.substring(1, word.length());
	}

}
