/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.perfs.basic;

import org.objectweb.jorm.perfs.JormBoot;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PBinderDelegate;
import org.objectweb.jorm.facility.meminst.manager.basic.api.Manager;
import org.objectweb.jorm.facility.naming.gobid.binder.LName;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.Properties;
import java.io.FileInputStream;
import util.Chrono;
import util.ListOfLong;
import util.ListOfLong;
import util.LongStat;

/**
 *
 * @author Sebastien Chassande-Barrioz, Xavier Spengler (for performance measurement)
 */
public class Application extends JormBoot {
    public final static byte CREATE_ACTION = (byte) 1;
    public final static byte LOAD_ACTION = (byte) 2;
    public final static byte REMOVE_ACTION = (byte) 3;

    static  public Chrono chrono = null;
    public ListOfLong lolRead = null;
    public LongStat longstatRead = null;
    public ListOfLong lolWrite = null;
    public LongStat longstatWrite = null;
    public ListOfLong lolRemove = null;
    public LongStat longstatRemove = null;
    static  ListOfLong lolRdbRead = null;
    static  ListOfLong lolRdbWrite = null;
    static  ListOfLong lolRdbRemove = null;

    public static void main(String[] args) {
	new Application().run(args);
    }

    public void run(String[] args) {

	chrono = new Chrono(true);
	lolRead = new ListOfLong();
	lolWrite = new ListOfLong();
	lolRemove = new ListOfLong();

	lolRdbRead = new ListOfLong();
	lolRdbRemove = new ListOfLong();
	lolRdbWrite = new ListOfLong();

	try {
	    if (args.length < 2) {
		printUsage();
		System.exit(-1);
	    }


	    Properties prop = null;

	    // Init the log system
	    prop = new Properties();
	    prop.load(new FileInputStream(args[1]));
	    initLogSystem(prop);

	    Logger readLogger = loggerFactory.getLogger("org.objectweb.jorm.result.read");
	    Logger writeLogger = loggerFactory.getLogger("org.objectweb.jorm.result.write");
	    Logger removeLogger = loggerFactory.getLogger("org.objectweb.jorm.result.remove");
	    Logger readRdbLogger = loggerFactory.getLogger("org.objectweb.jorm.result.rdb.read");
	    Logger writeRdbLogger = loggerFactory.getLogger("org.objectweb.jorm.result.rdb.write");
	    Logger removeRdbLogger = loggerFactory.getLogger("org.objectweb.jorm.result.rdb.remove");

	    // Init the jdbc driver
	    initResourceAdapter(prop);

	    for (byte action = CREATE_ACTION;
		 action < (REMOVE_ACTION + 1);
		 action++) {

		if (action == CREATE_ACTION)
		    initJorm(PClassMapping.CLEANUP_REMOVEALL);
		else
		    initJorm(PClassMapping.CLEANUP_DONOTHING);

		PName clientbinderpname =
		    lBinderMapping.decodeString(
						"org.objectweb.jorm.perfs.basic.Client");
		PBinderDelegate clientbinder = (PBinderDelegate)
		    ((Manager) lBinderMapping)
		    .getPBinding(conn, clientbinderpname)
		    .getPAccessor()
		    .getMemoryInstance();

		PClassMapping clientmapping = null;
		try {
		    clientmapping = (PClassMapping) Class.forName(
								  "org.objectweb.jorm.perfs.basic."
								  + mapperName + ".ClientMapping").newInstance();
		}
		catch(Exception e) {
		    logger.log(BasicLevel.ERROR,
			       "Impossible to get a new instance of ClientMapping",
			       e);
		    System.exit(-1);
		}

		clientmapping.setPBinder(clientbinder);
		clientbinder.setPClassMapping(clientmapping);
		if (action == CREATE_ACTION)
		    mapper.map(conn, clientmapping,
			       PClassMapping.CLEANUP_REMOVEALL);
		else
		    mapper.map(conn, clientmapping,
			       PClassMapping.CLEANUP_DONOTHING);

		for (int i = 2; i < args.length; i++) {
		    String arg = null;
		    if (action == CREATE_ACTION)
			arg = args[i];
		    else
			arg = "" + (i - 2);
		    PBinding pb1 = getClient(arg, action, clientmapping,
					     conn, logger);
		    //	System.out.println(pb1.getPAccessor().getMemoryInstance());
		    if (action == REMOVE_ACTION) {
			//Remove the client
			removeClient(clientmapping, pb1.getPName());
			logger.log(BasicLevel.DEBUG,
				   "Removing of the Client: Object Id = "
				   + ((LName) pb1.getPName()).getId());
		    }
		}
		
	    }
	    if (args[0].compareTo("read") == 0) {	    
		longstatRead = new LongStat(lolRead);
		readLogger.log(BasicLevel.INFO,"\n"+longstatRead.statisticalSummary("read", true));
		readLogger.log(BasicLevel.INFO,"\n"+lolRead.toString());
		if (lolRdbRead.size() != 0) {
		    LongStat longstatRdbRead = new LongStat(lolRdbRead);
		    readRdbLogger.log(BasicLevel.INFO,"\n"+longstatRdbRead.statisticalSummary("read", true));
		    longstatRdbRead.setLogger(readRdbLogger);
		    longstatRdbRead.outputSortedData();

		    longstatRdbRead.setLogger(readRdbLogger);
		    longstatRdbRead.frequency(20);
		    longstatRdbRead.outputFrequency(20);

		}

		longstatRead.setLogger(readLogger);
		longstatRead.frequency(20);
		longstatRead.outputFrequency(20);

	    }
	    if (args[0].compareTo("write") == 0) {	    
		longstatWrite = new LongStat(lolWrite);
		writeLogger.log(BasicLevel.INFO,"\n"+longstatWrite.statisticalSummary("write", true));
		writeLogger.log(BasicLevel.INFO,"\n"+lolWrite.toString());
		if (lolRdbWrite.size() != 0) {
		    LongStat longstatRdbWrite = new LongStat(lolRdbWrite);
		    writeRdbLogger.log(BasicLevel.INFO,"\n"+longstatRdbWrite.statisticalSummary("write", true));
		    longstatRdbWrite.setLogger(writeRdbLogger);
		    longstatRdbWrite.outputSortedData();

		    longstatRdbWrite.setLogger(writeRdbLogger);
		    longstatRdbWrite.frequency(20);
		    longstatRdbWrite.outputFrequency(20);

		}
		longstatWrite.setLogger(writeLogger);
		longstatWrite.frequency(20);
		longstatWrite.outputFrequency(20);


	    }
	    if (args[0].compareTo("remove") == 0) {	    
		longstatRemove = new LongStat(lolRemove);
		removeLogger.log(BasicLevel.INFO,"\n"+longstatRemove.statisticalSummary("remove", true));
		removeLogger.log(BasicLevel.INFO,"\n"+lolRemove.toString());
		if (lolRdbRemove.size() != 0) {
		    LongStat longstatRdbRemove = new LongStat(lolRdbRemove);
		    removeRdbLogger.log(BasicLevel.INFO,"\n"+longstatRdbRemove.statisticalSummary("remove", true));
		    longstatRdbRemove.setLogger(removeRdbLogger);
		    longstatRdbRemove.outputSortedData();

		    longstatRdbRemove.setLogger(removeRdbLogger);
		    longstatRdbRemove.frequency(20);
		    longstatRdbRemove.outputFrequency(20);

		}
		longstatRemove.setLogger(removeLogger);
		longstatRemove.frequency(20);
		longstatRemove.outputFrequency(20);

	    }

	}
	catch (Exception e) {
	    printStackTrace(e, logger);
	}
    }

    public PBinding getClient(String ln,
			      byte action,
			      PClassMapping clientmapping,
			      Object conn,
			      Logger logger) throws Exception {
	Client c = null;
	if (action == CREATE_ACTION)
	    c = new Client(ln);
	else
	    c = new Client("");
	PBinding pb = clientmapping.createPBinding();
	pb.setPAccessor(c);

	if (action == CREATE_ACTION) {
	    PName pnb = pb.export(conn);
	    c.identifier = pnb.encodeLong();
	    long t1 = chrono.time();
	    pb.write(conn, null);
	    lolWrite.add(chrono.time() - t1);
	    logger.log(BasicLevel.DEBUG, "Creation of a Client: Object Id = "
		       + ((LName) pnb).getId());
	}
	else {
	    pb.bind(clientmapping.decodeLong(Long.parseLong(ln)));
	    long t1 = chrono.time();
	    pb.read(conn, null);
	    lolRead.add(chrono.time() - t1);
	}
	return pb;
    }
    public void removeClient(PClassMapping clientMapping,
			     PName clientName) throws PException {
	// create instance of Client
	Client client = new Client(null);
	//Create a binding
	PBinding clientBinding = clientMapping.createPBinding();
	// link the binding to the memory instance
	clientBinding.setPAccessor(client);
	// link the binding the pname
	clientBinding.bind(clientName);
	// ask the remove of the data store instance
	clientBinding.unexport(conn);
	// Really remove the data store instance
	long t1 = chrono.time();
	clientBinding.write(conn, null);
	lolRemove.add(chrono.time() - t1);
	// Clean the binding
	clientBinding.unbind();
    }

    public static void printUsage() {
	System.out.println("Client <conf>  <lastName>*");
	System.out.println("\t<conf> : properties filename for init jorm>");
	System.out.println("\tYou must specify at least a last name");
    }

    public static void printStackTrace(Exception e, Logger logger) {
	if (e instanceof PException
	    && ((PException) e).getNestedException() != null) {

	    System.out.println(e.getMessage());
	    printStackTrace(((PException) e).getNestedException(), logger);
	}
	else {
	    if (logger != null)
		logger.log(BasicLevel.ERROR,"",e);
	    else
		e.printStackTrace();
	}
    }

    static public void storeDiffRead(long diff) {
	lolRdbRead.add(diff);
    }

    static public void storeDiffWrite(long diff) {
	lolRdbWrite.add(diff);
    }

    static public void storeDiffRemove(long diff) {
	lolRdbRemove.add(diff);
    }


}
